@@ -216,8 +216,7 @@ def main():
     )
     print(
         "   # Skip native build, since we have all needed native binaries from CI\n"
-        "   export MAVEN_SKIP_NATIVE_BUILD=1\n"
-        "   GPG_TTY=$(tty) mvn deploy -Prelease -DskipTests"
+        "   GPG_TTY=$(tty) mvn deploy -Prelease -DskipTests -Dskip.native.build=true"
     )
     print(
         "4. Log into https://oss.sonatype.org/. On the left menu panel, click Staging "
@@ -228,9 +227,8 @@ def main():
     )
     print(
         "5. Remove the Scala 2.12 artifacts and build Scala 2.13 artifacts:\n"
-        "   export MAVEN_SKIP_NATIVE_BUILD=1\n"
         "   python dev/change_scala_version.py --scala-version 2.13 --purge-artifacts\n"
-        "   GPG_TTY=$(tty) mvn deploy -Prelease-cpu-only,scala-2.13 -DskipTests"
+        "   GPG_TTY=$(tty) mvn deploy -Prelease -DskipTests -Dskip.native.build=true"
     )
     print(
         "6. Go to https://oss.sonatype.org/ to release the Scala 2.13 artifacts. "
