@@ -18,29 +18,39 @@ class LintersPaths:
         "python-package/",
         # tests
         "tests/python/test_config.py",
+        "tests/python/test_callback.py",
         "tests/python/test_data_iterator.py",
+        "tests/python/test_dmatrix.py",
         "tests/python/test_dt.py",
+        "tests/python/test_demos.py",
+        "tests/python/test_eval_metrics.py",
+        "tests/python/test_multi_target.py",
         "tests/python/test_predict.py",
         "tests/python/test_quantile_dmatrix.py",
         "tests/python/test_tree_regularization.py",
+        "tests/python/test_training_continuation.py",
         "tests/python/test_shap.py",
-        "tests/python-gpu/test_gpu_data_iterator.py",
-        "tests/python-gpu/test_gpu_prediction.py",
-        "tests/python-gpu/load_pickle.py",
-        "tests/python-gpu/test_gpu_pickling.py",
-        "tests/python-gpu/test_gpu_eval_metrics.py",
-        "tests/python-gpu/test_gpu_with_sklearn.py",
+        "tests/python/test_model_io.py",
+        "tests/python/test_with_pandas.py",
+        "tests/python-gpu/",
+        "tests/python-sycl/",
+        "tests/test_distributed/test_with_dask/",
+        "tests/test_distributed/test_gpu_with_dask/",
         "tests/test_distributed/test_with_spark/",
         "tests/test_distributed/test_gpu_with_spark/",
         # demo
         "demo/dask/",
         "demo/rmm_plugin",
         "demo/json-model/json_parser.py",
+        "demo/guide-python/continuation.py",
         "demo/guide-python/cat_in_the_dat.py",
         "demo/guide-python/callbacks.py",
         "demo/guide-python/categorical.py",
+        "demo/guide-python/cat_pipeline.py",
         "demo/guide-python/feature_weights.py",
         "demo/guide-python/sklearn_parallel.py",
+        "demo/guide-python/sklearn_examples.py",
+        "demo/guide-python/sklearn_evals_result.py",
         "demo/guide-python/spark_estimator_examples.py",
         "demo/guide-python/external_memory.py",
         "demo/guide-python/individual_trees.py",
@@ -77,16 +87,26 @@ class LintersPaths:
         "python-package/",
         # tests
         "tests/python/test_dt.py",
+        "tests/python/test_demos.py",
         "tests/python/test_data_iterator.py",
+        "tests/python/test_multi_target.py",
         "tests/python-gpu/test_gpu_data_iterator.py",
         "tests/python-gpu/load_pickle.py",
+        "tests/python-gpu/test_gpu_training_continuation.py",
+        "tests/python/test_model_io.py",
         "tests/test_distributed/test_with_spark/test_data.py",
         "tests/test_distributed/test_gpu_with_spark/test_data.py",
         "tests/test_distributed/test_gpu_with_dask/test_gpu_with_dask.py",
         # demo
+        "demo/dask/",
         "demo/json-model/json_parser.py",
         "demo/guide-python/external_memory.py",
+        "demo/guide-python/sklearn_examples.py",
+        "demo/guide-python/continuation.py",
+        "demo/guide-python/callbacks.py",
         "demo/guide-python/cat_in_the_dat.py",
+        "demo/guide-python/categorical.py",
+        "demo/guide-python/cat_pipeline.py",
         "demo/guide-python/feature_weights.py",
         "demo/guide-python/individual_trees.py",
         "demo/guide-python/quantile_regression.py",
