@@ -12,7 +12,6 @@ from xgboost.testing.params import (
     cat_parameter_strategy,
     exact_parameter_strategy,
     hist_cache_strategy,
-    hist_multi_parameter_strategy,
     hist_parameter_strategy,
 )
 from xgboost.testing.updater import (
@@ -25,69 +24,6 @@ from xgboost.testing.updater import (
 )
 
 
-class TestTreeMethodMulti:
-    @given(
-        exact_parameter_strategy, strategies.integers(1, 20), tm.multi_dataset_strategy
-    )
-    @settings(deadline=None, print_blob=True)
-    def test_exact(self, param: dict, num_rounds: int, dataset: tm.TestDataset) -> None:
-        if dataset.name.endswith("-l1"):
-            return
-        param["tree_method"] = "exact"
-        param = dataset.set_params(param)
-        result = train_result(param, dataset.get_dmat(), num_rounds)
-        assert tm.non_increasing(result["train"][dataset.metric])
-
-    @given(
-        exact_parameter_strategy,
-        hist_parameter_strategy,
-        hist_cache_strategy,
-        strategies.integers(1, 20),
-        tm.multi_dataset_strategy,
-    )
-    @settings(deadline=None, print_blob=True)
-    def test_approx(
-        self, param: Dict[str, Any],
-        hist_param: Dict[str, Any],
-        cache_param: Dict[str, Any],
-        num_rounds: int,
-        dataset: tm.TestDataset,
-    ) -> None:
-        param["tree_method"] = "approx"
-        param = dataset.set_params(param)
-        param.update(hist_param)
-        param.update(cache_param)
-        result = train_result(param, dataset.get_dmat(), num_rounds)
-        note(str(result))
-        assert tm.non_increasing(result["train"][dataset.metric])
-
-    @given(
-        exact_parameter_strategy,
-        hist_multi_parameter_strategy,
-        hist_cache_strategy,
-        strategies.integers(1, 20),
-        tm.multi_dataset_strategy,
-    )
-    @settings(deadline=None, print_blob=True)
-    def test_hist(
-        self,
-        param: Dict[str, Any],
-        hist_param: Dict[str, Any],
-        cache_param: Dict[str, Any],
-        num_rounds: int,
-        dataset: tm.TestDataset,
-    ) -> None:
-        if dataset.name.endswith("-l1"):
-            return
-        param["tree_method"] = "hist"
-        param = dataset.set_params(param)
-        param.update(hist_param)
-        param.update(cache_param)
-        result = train_result(param, dataset.get_dmat(), num_rounds)
-        note(str(result))
-        assert tm.non_increasing(result["train"][dataset.metric])
-
-
 class TestTreeMethod:
     USE_ONEHOT = np.iinfo(np.int32).max
     USE_PART = 1
@@ -99,10 +35,24 @@ class TestTreeMethod:
     def test_exact(self, param, num_rounds, dataset):
         if dataset.name.endswith("-l1"):
             return
-        param['tree_method'] = 'exact'
+        param["tree_method"] = "exact"
         param = dataset.set_params(param)
         result = train_result(param, dataset.get_dmat(), num_rounds)
-        assert tm.non_increasing(result['train'][dataset.metric])
+        assert tm.non_increasing(result["train"][dataset.metric])
+
+    def test_exact_sample_by_node_error(self) -> None:
+        X, y, w = tm.make_regression(128, 12, False)
+        with pytest.raises(ValueError, match="column sample by node"):
+            xgb.train(
+                {"tree_method": "exact", "colsample_bynode": 0.999},
+                xgb.DMatrix(X, y, weight=w),
+            )
+
+        xgb.train(
+            {"tree_method": "exact", "colsample_bynode": 1.0},
+            xgb.DMatrix(X, y, weight=w),
+            num_boost_round=2,
+        )
 
     @given(
         exact_parameter_strategy,
@@ -181,7 +131,6 @@ class TestTreeMethod:
         ag_param = {'max_depth': 2,
                     'tree_method': 'hist',
                     'eta': 1,
-                    'verbosity': 0,
                     'objective': 'binary:logistic',
                     'eval_metric': 'auc'}
         hist_res = {}
@@ -404,7 +353,8 @@ class TestTreeMethod:
 
         assert get_score(config_0) == get_score(config_1)
 
-        raw_booster = booster_1.save_raw(raw_format="deprecated")
+        with pytest.warns(Warning, match="Model format is default to UBJSON"):
+            raw_booster = booster_1.save_raw(raw_format="deprecated")
         booster_2 = xgb.Booster(model_file=raw_booster)
         config_2 = json.loads(booster_2.save_config())
         assert get_score(config_1) == get_score(config_2)
